@@ -117,9 +117,8 @@ static sig_atomic_t signaled;
 const char *initddir = "/etc/init.d";
 const char *etcdir = "/etc";
 
-#define LEGACY_DEPENDENCY_PATH "/etc/init.d/."
-/* #define DEPENDENCY_PATH "/lib/insserv/" */
-char *dependency_path = LEGACY_DEPENDENCY_PATH;
+#define WRITE_TO_FD 2
+
 #ifndef PATH_MAX
 #define PATH_MAX 2048
 #endif
@@ -150,6 +149,7 @@ struct prg {
 static struct prg *prgs;
 static int inpar, par;
 static double iorate = 800.0;
+int print_program_name = FALSE;
 
 #ifdef USE_PRELOAD
 static volatile enum { Unknown, Preload, NoPreload } ispreload = Unknown;
@@ -296,16 +296,37 @@ void writebuf(struct prg *p)
   char *b = p->buf;
   int r;
 
+  #ifdef DEBUG
+  fprintf(stderr, "About to write buf\n");
+  #endif
+  if ( (print_program_name) && (p->name) && (p->len > 0) )
+  { 
+     char *temp_name;
+     int status;
+
+     status = asprintf(&temp_name, "%s: ", p->name);
+     if (status >= 0)
+     {
+        write(WRITE_TO_FD, temp_name, status);
+        free(temp_name);
+     }
+  }
   while (p->len > 0)
     {
-      r = write(2, b, p->len);
+      r = write(WRITE_TO_FD, b, p->len);
       if (r < 0)
-	{
+      {
 	  if (errno == EINTR)
 	    continue;
 	  perror("write");
 	  r = p->len;
-	}
+      }
+      else
+      {
+          /* make sure we finish output with a newline */
+          if (b[p->len - 1] != '\n')
+             write(WRITE_TO_FD, "\n", 2);
+      }
       p->len -= r;
       b += r;
     }
@@ -360,7 +381,6 @@ static int checksystem(const int par, const boolean start, const boolean limit)
 
 #if DEBUG
   fprintf(stderr, "checksystem par=%d newpar=%d (prcs_run=%lu) %ld\n", par, newpar, prcs_run, time(0));
-  dump_status();
 #endif
   if (newpar <= 0)
     return 1;
@@ -421,15 +441,24 @@ static int checkdevpts(void)
 
   if (ptsfd == -1)
     {
+      #ifdef DEBUG
+      fprintf(stderr, "No PT found\n");
+      #endif
       return 0;
     }
   else if (ptsname(ptsfd) == 0 || grantpt(ptsfd) || unlockpt(ptsfd))
     {
+      #ifdef DEBUG
+      fprintf(stderr, "Error getting PT\n");
+      #endif
       close(ptsfd);
       return 0;
     }
   else
     {
+      #ifdef DEBUG
+      fprintf(stderr, "PT found\n");
+      #endif
       close(ptsfd);
       return 1;
     }
@@ -602,6 +631,10 @@ int run_single(const char *prg, const char *arg0, int spl)
   pid_t pid;
   int r;
 
+  #ifdef DEBUG
+  fprintf(stderr, "Running interactive, single task.\n");
+  #endif
+
   if ((pid = fork()) == (pid_t)-1)
     {
       perror("fork");
@@ -687,10 +720,10 @@ static size_t gtimo_buflen;
 void storebuf(struct prg *p)
 {
   if ((gtimo_buflen + p->len) > gtimo_bufsize)
-    {
+  {
       writebuf(p);				/* In case of overflow or memory shortage */
       return;
-    }
+  }
 
   (void)memcpy(gtimo_buf + gtimo_buflen, p->buf, p->len);
   gtimo_buflen += p->len;
@@ -698,7 +731,7 @@ void storebuf(struct prg *p)
   glastio = now;
 }
 
-void flushbuf(void)
+void flushbuf(struct prg *p)
 {
   size_t len = gtimo_buflen;
   char * buf = gtimo_buf;
@@ -706,14 +739,34 @@ void flushbuf(void)
   if (!buf)
 	return;					/* In case of memory shortage */
 
+  #ifdef DEBUG
+  fprintf(stderr, "About to flush buf\n");
+  #endif
+  if ( (print_program_name) && (len > 0) && (p->name) )
+  {
+      char *temp_name;
+      int status;
+      status = asprintf(&temp_name, "%s: ", p->name);
+      if (status >= 0)
+      {
+          write(WRITE_TO_FD, temp_name, status);
+          free(temp_name);
+      }
+  }
   while (len > 0)
     {
-      int r = write(2, buf, len);
+      int r = write(WRITE_TO_FD, buf, len);
       if (r < 0)
-	{
+      {
 	  perror("write");
 	  r = len;
-	}
+      }
+      else
+      {
+         /* make sure we finish output with a newline */
+         if (buf[len - 1] != '\n')
+            write(WRITE_TO_FD, "\n", 2);
+      }
       len -= r;
       buf += r;
     }
@@ -789,7 +842,7 @@ void detach(struct prg *p, const int store)
   p->fd = 0;
 }
 
-static struct prg *interactive_task;
+static struct prg *interactive_task = NULL;
 static volatile int active;
 static void sigchld(int sig attribute((unused)))
 {
@@ -865,6 +918,7 @@ void usage(int status)
   fprintf(stderr, "       -l use legacy /etc/init.d path for Makefile-style scripts\n");
   fprintf(stderr, "          The default is to use the location /lib/insserv\n");
   */
+  fprintf(stderr, "       -n prefix output with name of program displaying output\n");
   fprintf(stderr, "       -p parallel tasks\n");
   fprintf(stderr, "       -t I/O timeout\n");
   fprintf(stderr, "       -T global I/O timeout\n");
@@ -891,7 +945,6 @@ int main(int argc, char **argv)
   char *splashopt = 0;
   sigset_t nmask, omask, smask;
   int status;     /* check if parsing makefile worked */
-  int using_legacy_path = FALSE;      /* use new makefile path by default */
 
   detect_consoles();
 
@@ -904,10 +957,13 @@ int main(int argc, char **argv)
   numcpu = sysconf(_SC_NPROCESSORS_ONLN);
   myname = argv[0];
 
-  while ((c = getopt(argc, argv, "fhp:t:T:a:M:P:R:S:vi:e:d:")) != EOF)
+  while ((c = getopt(argc, argv, "fhnp:t:T:a:M:P:R:S:vi:e:d:")) != EOF)
     {
       switch(c)
         {
+        case 'n':
+          print_program_name = TRUE;
+          break;
 	case 'p':
 	  par = atoi(optarg);
 	  break;
@@ -929,12 +985,6 @@ int main(int argc, char **argv)
         case 'e':
           etcdir = optarg;
           break;
-        /*
-        case 'l':
-          dependency_path = LEGACY_DEPENDENCY_PATH;
-          using_legacy_path = TRUE;
-          break;
-        */
 	case 'M':
 	  run_mode = optarg;
 	  break;
@@ -950,17 +1000,15 @@ int main(int argc, char **argv)
 	case 'v':
 	  printf("startpar version %s\n", VERSION);
 	  exit(0);
-	case 'h':
-	  usage(0);
-	  break;
 	case 'i':
 	  iorate = atof(optarg);
 	  if (iorate < 0.0)
 	    iorate = 800.0;
 	  break;
+	case 'h':
+	  usage(0);     /* usage does not return, no need for break */
 	default:
 	  usage(1);
-	  break;
         }
     }
   if (forw)
@@ -1006,21 +1054,14 @@ int main(int argc, char **argv)
 	  fprintf(stderr, "invalid run mode %s\n", run_mode);
 	  exit(1);
 	}
-      snprintf(makefile, PATH_MAX, "%sdepend.%s", dependency_path, run_mode);
+      snprintf(makefile, PATH_MAX, "%s/.depend.%s", initddir, run_mode);
       status = parse_makefile(makefile);
-      /* If the parse fails it is probably because the file does not
-         exist. Try alternative location. -- Jesse */
-      if ( (! status) && (! using_legacy_path) )
+      if (! status) 
       {
-         snprintf(makefile, PATH_MAX, "%sdepend.%s", LEGACY_DEPENDENCY_PATH, run_mode);
-         fprintf(stderr, "Trying to find makefile-style script at legacy location: %s\n",
-                         LEGACY_DEPENDENCY_PATH);
-         status = parse_makefile(makefile);
-         if (! status)
-            exit(1);
-      }
-      else if ( (! status) && (using_legacy_path) )
+         fprintf(stderr, "Unable to run scripts using dependency information in %s\n",
+                 initddir);
          exit(1);      /* no other place to try so drop out */
+      }
 
       check_run_files(run_mode, prev_level, run_level);
 
@@ -1035,7 +1076,7 @@ int main(int argc, char **argv)
       if (par > argc)				/* not more than the number of all scripts */
 	par = argc;
 
-      inpar = par;				/* the original argument of parallel procs per cpu */
+      inpar = par;			/* the original argument of parallel procs per cpu */
 
       par = checkpar(inpar, isstart);		/* the number of parallel procs on all cpu's */
 
@@ -1079,11 +1120,19 @@ int main(int argc, char **argv)
 	{
 	  if ((*nodevec = pickup_task()))
 	  {
+            #ifdef DEBUG
+            fprintf(stderr, "About to run single task in run mode %s\n", run_mode);
+            #endif
 	    *resvec = run_single((*nodevec)->name, (*nodevec)->arg0, calcsplash(0, 1, splashopt));
 	    finish_task(*nodevec);
 	  }
       } else
-	*resvec = run_single(*argv, *argv, calcsplash(0, 1, splashopt));
+        {
+            #ifdef DEBUG
+            fprintf(stderr, "About to run single task with no devec\n");
+            #endif
+	   *resvec = run_single(*argv, *argv, calcsplash(0, 1, splashopt));
+        }
       goto finished;
     }
 
@@ -1184,7 +1233,7 @@ int main(int argc, char **argv)
   for (;;)
     {
 #ifdef CHECK_FORDEVPTS
-      int devpts = 0;
+      int devpts = checkdevpts();
 #endif
       int maxfd = -1;
       int last = -1;
@@ -1244,10 +1293,20 @@ int main(int argc, char **argv)
 		  p->num = num++;
 #ifdef CHECK_FORDEVPTS
 		  if (!devpts)
-		    interactive_task = p;	/* no /dev/pts, treat as interactive */
+                  {
+                      #ifdef DEBUG
+                      fprintf(stderr, "No devpts. Setting interactive.\n");
+                      #endif
+		      interactive_task = p;	/* no /dev/pts, treat as interactive */
+                  }
 #endif
 		  if (notty)
+                  {
+                    #ifdef DEBUG
+                    fprintf(stderr, "No tty available. Set as interactive.\n");
+                    #endif
 		    interactive_task = p;	/* no tty, treat as interactive */
+                  }
 		  if (interactive_task)
 		    continue;			/* don't start this here */
 		  run(p);
@@ -1294,6 +1353,9 @@ int main(int argc, char **argv)
 	    {
 	      p = interactive_task;
 	      p->flags |= PRUNNING;
+              #ifdef DEBUG
+              fprintf(stderr, "About to run single, interactive task, active zero.\n");
+              #endif
 	      resvec[p->num] = run_single(p->name, p->arg0, p->splashadd);
 	      if (run_mode)
 		finish_task(nodevec[p->num]);
@@ -1373,7 +1435,7 @@ int main(int argc, char **argv)
 		{
 		  writebuf(p);
 		  if (p->fd) detach(p, GTIMO_OFFL);
-		  flushbuf();
+		  flushbuf(p);
 		  gtimo_running = 0;
 		}
 	      else if (gtimo_running)
@@ -1393,7 +1455,12 @@ int main(int argc, char **argv)
 
 #ifdef CHECK_FORDEVPTS
 	  if (!devpts)
-	    devpts = checkdevpts();
+          {
+	      devpts = checkdevpts();
+              #ifdef DEBUG
+              fprintf(stderr, "Check for devpts returned %d\n", devpts);
+              #endif
+          }
 #endif
 	  continue;		/* start new processes */
 	}
